/**************************************************************************//**
 * @file     IR_distance_detect.c
 * @version  V1.00
 * $Revision:
 * $Date: 14/08/15 6:09p $
 * @brief    Detect distance by IR function source file
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include "M0518.h"
#include "function.h"
#include "Action_Script.h"

#define     MaxValue            0xFFFF

/*---------------------------------------------------------------------------------------------------------*/
/* lrDA buffer																																														 */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t au8IR_CODE[4];

void IR_Init(void)
{

		/* Set GPF.4 multi-function pins for PWM1 Channel 4 to output IrDA code*/
		SYS->GPF_MFP &= ~(SYS_GPF_MFP_PF4_Msk);
		SYS->GPF_MFP |= (SYS_GPF_MFP_PF4_PWM1_CH4);
		SYS->ALT_MFP3	&= ~(SYS_ALT_MFP3_PF4_Msk);
		SYS->ALT_MFP3	|= (SYS_ALT_MFP3_PF4_PWM1_CH4);
	
		/* Set GPA.10 multi-function pins for PWM1 Channel 2 to capture IrDA code */
		SYS->GPA_MFP &= ~SYS_GPA_MFP_PA10_Msk;
		SYS->GPA_MFP |= SYS_GPA_MFP_PA10_PWM1_CH2;
		SYS->ALT_MFP3	&= ~SYS_ALT_MFP3_PA10_Msk;
		SYS->ALT_MFP3	|= SYS_ALT_MFP3_PA10_PWM1_CH2;

	
		/* set PWM1 channel 4 output configuration for IR generator with 38kHz */
		PWM_SET_PRESCALER(PWM1, 4, 1);
		PWM_SET_CNR(PWM1, 4, 50000000/2/38000);
		PWM_SET_CMR(PWM1, 4, 50000000/2/38000/2);
	
		/* set PWM1 channel 2 capture configuration for IR capture */		
		PWM_SET_PRESCALER(PWM1, 2, 49);
		PWM_SET_CNR(PWM1, 2, MaxValue);
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init Timer Clock                                                                                  */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Enable peripheral clock */
		CLK_EnableModuleClock(TMR2_MODULE);

    /* Select peripheral clock source */
		CLK_SetModuleClock(TMR2_MODULE, CLK_CLKSEL1_TMR2_S_HCLK, 0);

    /* Reset Funcction */
		SYS_ResetModule(TMR2_RST);

		/*---------------------------------------------------------------------------------------------------------*/
		/* lrDA Code																																															 */
		/*---------------------------------------------------------------------------------------------------------*/
		au8IR_CODE[0] = 0x00;
		au8IR_CODE[1] = ~au8IR_CODE[0];
		au8IR_CODE[2] = 0x10;
		au8IR_CODE[3] = ~au8IR_CODE[2];
	
}

void IR_Enable(void)
{
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Enable IR distance detect function                                                                      */
    /*---------------------------------------------------------------------------------------------------------*/
		/* Start PWM1 channel 2_3 and channel 4_5 counters */
		PWM_Start(PWM1, 0x14);
	
		/* Enable PWM1 channel 4 output */		
		PWM_EnableOutput(PWM1, 0x10);
	
		/* Enable PWM1 channel 2 capture and interrupt */
		PWM_EnableCapture(PWM1, 0x04);
	  PWM1->CAPCTL |= 0x04000000;
		PWM1->CTL1 |= 0x10;
    PWM_EnableCaptureInt(PWM1, 2, PWM_CAPTURE_INT_FALLING_LATCH);
    NVIC_EnableIRQ(PWM1_IRQn);
		NVIC_SetPriority(PWM1_IRQn, 0);
	
		/* Open Timer2 in periodic mode, enable interrupt and 1 interrupt ticks per second */
	  TIMER_Open(TIMER2, TIMER_PERIODIC_MODE, 1);
    TIMER_EnableInt(TIMER2);

    /* Enable Timer2 NVIC */
    NVIC_EnableIRQ(TMR2_IRQn);	
		NVIC_SetPriority(TMR2_IRQn, 3);

    /* Start Timer2 counting */
    TIMER_Start(TIMER2);
		
}

void IR_Disable(void)
{

		/*---------------------------------------------------------------------------------------------------------*/
    /* Disable IR distance detect function                                                                     */
    /*---------------------------------------------------------------------------------------------------------*/
		/* Disable PWM1 channel 4 output */		
		PWM_DisableOutput(PWM1, 0x10);
	
		/* Disable PWM1 channel 2 capture and interrupt */
		PWM_DisableCapture(PWM1, 0x04);
    PWM_DisableCaptureInt(PWM1, 2, PWM_CAPTURE_INT_FALLING_LATCH);
    NVIC_DisableIRQ(PWM1_IRQn);
}

#define 	Percent 					0.04		// 4 % 

// Leader code range
// LDC_MAX = 13460 * (1 + Percent) / 400 
#define 	IR_LDC_MAX 				(13460 * (1 + Percent)) 	
#define 	IR_LDC_MIN 				(13460 * (1 - Percent)) 	
// Repeater code range
#define 	IR_RPC_MAX 				(11280 * (1 + Percent)) 	
#define 	IR_RPC_MIN 				(11280 * (1 - Percent)) 
// Bit = 1 range
#define 	IR_BIT_1_MAX 			(2236 * (1 + Percent)) 
#define 	IR_BIT_1_MIN			(2236 * (1 - Percent)) 
// Bit = 0 range
#define 	IR_BIT_0_MAX 			(1120 * (1 + Percent)) 
#define 	IR_BIT_0_MIN			(1120 * (1 - Percent)) 

volatile uint8_t 			IR_State = 0;			// IR State
volatile uint8_t 			IR_LDC_Ready = 0;	// LeaDer Code is Ready
volatile uint8_t 			IR_CTC_Ready = 0;	// CusTomer Code is Ready
volatile uint8_t 			IR_CTC0 = 0;			// Received CusTomer Code 0
volatile uint8_t 			IR_CTC1 = 0;			// Received CusTomer Code 1
volatile uint8_t  		IR_DAC = 0;				// Received DAta Code
volatile uint8_t  		IR_DAB = 0;				// Received DAta Bar code
volatile uint8_t  		IR_cnt = 0;	
         uint8_t			IR_CODE[4]	=	{0x00, 0x00, 0x00, 0x00};

/*---------------------------------------------------------------------------------------------------------*/
/* Receive NEC IR		                                                                  										 */
/*---------------------------------------------------------------------------------------------------------*/					
void IrDa_NEC_Rx(uint32_t u32Time)
{
		uint32_t count;
// 		uint32_t Duration = u32Time;
// 		uint32_t temp = Duration; 
	
		if(IR_State == 0)
		{
				IR_LDC_Ready = 0;		  						// Clear LeaDer Code Ready
				IR_CTC_Ready = 0;									// Clear CusTomer Code Ready
				IR_State++;
		}			
		// Leader or Repeater code
		else if(IR_State == 1)
		{  
				// Leader code
				if((u32Time >= IR_LDC_MIN) && (u32Time <= IR_LDC_MAX))
				{	
						IR_LDC_Ready = 1;							// Set LeaDer Code Ready									
						IR_State++;		
				}
				else
				{
						IR_State = 1;
            IR_LDC_Ready = 0;		  				// Clear LeaDer Code Ready
            IR_CTC_Ready = 0;							// Clear CusTomer Code Ready           
				}
		}   
		// Customer code 0
		else if((IR_State >= 2 && IR_State < 10) && (IR_LDC_Ready == 1))
		{	
        IR_State++;
				IR_CTC0 = IR_CTC0 >> 1;
				if((u32Time >= IR_BIT_0_MIN) && (u32Time <= IR_BIT_0_MAX))      	// 1.12ms = 0 	
						IR_CTC0 &= 0x7f;
				else if((u32Time >= IR_BIT_1_MIN) && (u32Time <= IR_BIT_1_MAX))		// 2.25ms = 1 
						IR_CTC0 |= 0x80;
        else
            IR_State = 0;
		}   
		// Customer code 1
		else if((IR_State >= 10 && IR_State < 18) && (IR_LDC_Ready == 1)){
				IR_State++;        
				IR_CTC1 = IR_CTC1 >> 1;
				if((u32Time >= IR_BIT_0_MIN) && (u32Time <= IR_BIT_0_MAX))      	// 1.12ms = 0 	
						IR_CTC1 &= 0x7f;
				else if((u32Time >= IR_BIT_1_MIN) && (u32Time <= IR_BIT_1_MAX))		// 2.25ms = 1 
						IR_CTC1 |= 0x80;  	
        else
            IR_State = 0;								
		}  
		// Data 
		else if((IR_State >= 18 && IR_State < 26) && (IR_LDC_Ready == 1))
		{
				IR_State++;        
				IR_DAC = IR_DAC >> 1;										
				if((u32Time >= IR_BIT_0_MIN) && (u32Time <= IR_BIT_0_MAX))      	// 1.12ms = 0 
						IR_DAC &= 0x7f;
				else if((u32Time >= IR_BIT_1_MIN) && (u32Time <= IR_BIT_1_MAX)) 	// 2.25ms = 1 
						IR_DAC |= 0x80;  	
        else
            IR_State = 0;								

		} 
		// Data bar
		else if((IR_State >= 26 && IR_State < 34) && (IR_LDC_Ready == 1))
		{	
        IR_State++;
				IR_DAB = IR_DAB >> 1;										
				if((u32Time >= IR_BIT_0_MIN) && (u32Time <= IR_BIT_0_MAX)) 	  		// 1.12ms = 0	
						IR_DAB &= 0x7f;
				else if((u32Time >= IR_BIT_1_MIN) && (u32Time <= IR_BIT_1_MAX)) 	// 2.25ms = 1 
						IR_DAB |= 0x80;  	
        else
            IR_State = 0;					

				if(IR_State == 34)
				{
						if((IR_DAC ^ IR_DAB) == 0xff)
						{
								IR_LDC_Ready = 0;			// Clear LeaDer Code Ready 
								IR_CODE[0] = IR_CTC0;
								IR_CODE[1] = IR_CTC1;
								IR_CODE[2] = IR_DAC;
								IR_CODE[3] = IR_DAB; 
								IR_cnt++;

								// Detect obstacle and then walk backward 
								if((IR_CODE[0] == 0x00)& (IR_CODE[1] == 0xFF))
								{
										if((IR_CODE[2] == 0x10)& (IR_CODE[3] == 0xEF))
										{
												Action_Init2();
												Buzzer_Pulse(250000);
												Buzzer_Pulse(250000);
												for(count=0; count<10; count++)
														Walk_Backward_TouchDown();
												Walk_Backward_TouchUp();
										}
								}

						}
						IR_State = 0;	
				}	
		}    
}

void PWM1_IRQHandler(void)
{
		//uint8_t	CAPIF_tmp;
    uint32_t TDR1_tmp;
    TDR1_tmp = MaxValue - PWM_GET_CAPTURE_FALLING_DATA(PWM1, 2);
		//CAPIF_tmp = PWM_GetCaptureIntFlag(PWM1, 2);
		PWM_ClearCaptureIntFlag(PWM1, 2, PWM_CAPTURE_INT_FALLING_LATCH | PWM_CAPTURE_INT_RISING_LATCH);
    //if(CAPIF_tmp == 0x0400)
			IrDa_NEC_Rx(TDR1_tmp);

}

#define		NEC_LDC_MARK			16 		// 16 x 560us = 8960us =   9ms
#define 	NEC_LDC_SPACE			8		//  8 x 560us = 4480us = 4.5ms
#define 	NEC_BIT_MARK			1		// 560us
#define 	NEC_ONE_SPACE			3		//  3 x 560us = 1680us = 1690us
#define 	NEC_ZERO_SPACE		1		// 560us
#define 	NEC_BYTES					4

/*---------------------------------------------------------------------------------------------------------*/
/* Mark                                                                  																   */
/*																																																				 */
/* Pulse = 1/3 duty @38KHz frequency																																			 */
/*---------------------------------------------------------------------------------------------------------*/ 
void Mark(uint8_t N)
{
    /* Switch to PWM output waveform */
    PWM_EnableOutput(PWM1, 0x10);
    CLK_SysTickDelay(560*N);
    PWM_DisableOutput(PWM1, 0x10);
}

void SPACE(uint8_t N)
{
    CLK_SysTickDelay(560*N);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Send NEC IR Format                                                                  										 */
/*---------------------------------------------------------------------------------------------------------*/
void SendNEC(uint8_t* data)
{
		uint8_t	nbyte;
		uint8_t	nbit;

		/* Send out Leader code */
		Mark(NEC_LDC_MARK);
		SPACE(NEC_LDC_SPACE);
   
		/* Send out Customer code and Data code	*/
		for (nbyte=0; nbyte < NEC_BYTES; nbyte++){
			for (nbit=0; nbit < 8; nbit++){
				Mark(NEC_BIT_MARK);
						if (data[nbyte] & (1 << nbit))		// LSB first
							SPACE(NEC_ONE_SPACE);
						else
							SPACE(NEC_ZERO_SPACE); 
			}
		}

		/* Send out Stop bit */
		Mark(NEC_BIT_MARK); 

} 

void TMR2_IRQHandler(void)
{

    if(TIMER_GetIntFlag(TIMER2) == 1)
    {
        /* Clear Timer2 time-out interrupt flag */
        TIMER_ClearIntFlag(TIMER2);

				SendNEC(au8IR_CODE);
    }
		
}
/*** (C) COPYRIGHT 2014 Nuvoton Technology Corp. ***/
