/**************************************************************************//**
 * @file     main.c
 * @version  V2.00
 * $Revision: 4 $
 * $Date: 14/06/30 4:51p $
 * @brief    Demo for Six-foot Multi-axis Robot.
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "M0518.h"
#include "action_script.h"
#include "function.h"

#define PLL_CLOCK       50000000
#define buffsize				4
#define BT_LED					PC6					// BLUETOOTH_CONNECTION_LED
#define LowB_LED				PB2					// LOW_BATTERY_LED < Batlow1
#define LowP_LED				PB3					// LOW_POWER_LED < CHIPLOW1
#define On							0
#define Off							1

/*-----------------------------------------------------------------*/
/* Global Variables                                                */
/*-----------------------------------------------------------------*/
uint8_t buff[buffsize] = {0};
uint8_t Command;
uint8_t Bluetooth;
extern uint8_t au8IR_CODE[4];
void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable Internal RC clock */
    CLK_EnableXtalRC(CLK_PWRCON_IRC22M_EN_Msk);

    /* Waiting for IRC22M clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_IRC22M_STB_Msk);

    /* Switch HCLK clock source to Internal RC and HCLK source divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_CLKDIV_HCLK(1));

    /* Enable PLL and Set PLL frequency */
    CLK_SetCoreClock(PLL_CLOCK);
	  CLK_SetSysTickClockSrc(CLK_CLKSEL0_STCLK_S_HCLK_DIV2);

    /* Waiting for clock ready */
		CLK_WaitClockReady(CLK_CLKSTATUS_PLL_STB_Msk | CLK_CLKSTATUS_IRC22M_STB_Msk);

    /* Switch HCLK clock source to PLL, STCLK to HCLK/2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_PLL, CLK_CLKDIV_HCLK(2));

    /* Enable peripheral clock */
    CLK_EnableModuleClock(UART0_MODULE);
    CLK_EnableModuleClock(PWM0_MODULE);
    CLK_EnableModuleClock(PWM1_MODULE);
		CLK_EnableModuleClock(BPWM0_MODULE);
    CLK_EnableModuleClock(BPWM1_MODULE);
		CLK_EnableModuleClock(TMR0_MODULE);
		CLK_EnableModuleClock(TMR1_MODULE);
		CLK_EnableModuleClock(ADC_MODULE);

    /* Select peripheral clock source */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART_S_HIRC, CLK_CLKDIV_UART(1));
		CLK_SetModuleClock(PWM0_MODULE, CLK_CLKSEL3_PWM0_S_PLL, 0);
		CLK_SetModuleClock(PWM1_MODULE, CLK_CLKSEL3_PWM1_S_PLL, 0);	
		CLK_SetModuleClock(BPWM0_MODULE, CLK_CLKSEL3_BPWM0_S_PLL, 0);
		CLK_SetModuleClock(BPWM1_MODULE, CLK_CLKSEL3_BPWM1_S_PLL, 0);	
		CLK_SetModuleClock(TMR0_MODULE, CLK_CLKSEL1_TMR0_S_HCLK, 0);
		CLK_SetModuleClock(TMR1_MODULE, CLK_CLKSEL1_TMR1_S_HCLK, 0);
		CLK_SetModuleClock(ADC_MODULE, CLK_CLKSEL1_ADC_S_HIRC, CLK_CLKDIV_ADC(7));

    /* Reset Funcction */
		SYS_ResetModule(UART0_RST);
    SYS_ResetModule(PWM0_RST);
		SYS_ResetModule(PWM1_RST);
		SYS_ResetModule(BPWM0_RST);
		SYS_ResetModule(BPWM1_RST);
		SYS_ResetModule(TMR0_RST);
		SYS_ResetModule(TMR1_RST);
    SYS_ResetModule(ADC_RST);

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set GPB multi-function pins for UART0 RXD and TXD */
    SYS->GPB_MFP &= ~(SYS_GPB_MFP_PB0_Msk | SYS_GPB_MFP_PB1_Msk);
    SYS->GPB_MFP |= (SYS_GPB_MFP_PB0_UART0_RXD | SYS_GPB_MFP_PB1_UART0_TXD);
		
    /* Set GPA.12/13/14/15/0/1 multi-function pins for PWM0 Channel 0 ~ channel 5 */
    SYS->GPA_MFP &= ~(SYS_GPA_MFP_PA12_Msk | SYS_GPA_MFP_PA13_Msk | SYS_GPA_MFP_PA14_Msk | SYS_GPA_MFP_PA15_Msk | SYS_GPA_MFP_PA0_Msk | SYS_GPA_MFP_PA1_Msk);
    SYS->GPA_MFP |= (SYS_GPA_MFP_PA12_PWM0_CH0 | SYS_GPA_MFP_PA13_PWM0_CH1 | SYS_GPA_MFP_PA14_PWM0_CH2 | SYS_GPA_MFP_PA15_PWM0_CH3 | SYS_GPA_MFP_PA0_PWM0_CH4 | SYS_GPA_MFP_PA1_PWM0_CH5);
    SYS->ALT_MFP3 &= ~(SYS_ALT_MFP3_PA0_Msk | SYS_ALT_MFP3_PA1_Msk);
		SYS->ALT_MFP3 |= (SYS_ALT_MFP3_PA0_PWM0_CH4 | SYS_ALT_MFP3_PA1_PWM0_CH5);
		SYS->ALT_MFP4 &= ~(SYS_ALT_MFP4_PA12_Msk | SYS_ALT_MFP4_PA13_Msk | SYS_ALT_MFP4_PA0_Msk | SYS_ALT_MFP4_PA1_Msk);
		
		/* Set GPA.2/3 multi-function pins for PWM1 Channel 0 ~ channel 1 */
    SYS->GPA_MFP &= ~(SYS_GPA_MFP_PA2_Msk | SYS_GPA_MFP_PA3_Msk);
		SYS->GPA_MFP |= (SYS_GPA_MFP_PA2_PWM1_CH0 | SYS_GPA_MFP_PA3_PWM1_CH1);
		SYS->ALT_MFP3	&= ~(SYS_ALT_MFP3_PA2_Msk | SYS_ALT_MFP3_PA3_Msk);
		SYS->ALT_MFP3	|= (SYS_ALT_MFP3_PA2_PWM1_CH0 | SYS_ALT_MFP3_PA3_PWM1_CH1);
		SYS->ALT_MFP4 &= ~(SYS_ALT_MFP4_PA2_Msk | SYS_ALT_MFP4_PA3_Msk);
		
		/* Set GPC.0/1/2/3 and GPD.15/14 multi-function pins for BPWM0 Channel 0 ~ channel 5 */
    SYS->GPC_MFP &= ~(SYS_GPC_MFP_PC0_Msk | SYS_GPC_MFP_PC1_Msk | SYS_GPC_MFP_PC2_Msk | SYS_GPC_MFP_PC3_Msk);
		SYS->GPC_MFP |= (SYS_GPC_MFP_PC0_BPWM0_CH0 | SYS_GPC_MFP_PC1_BPWM0_CH1 | SYS_GPC_MFP_PC2_BPWM0_CH2 | SYS_GPC_MFP_PC3_BPWM0_CH3);
		SYS->GPD_MFP &= ~(SYS_GPD_MFP_PD15_Msk | SYS_GPD_MFP_PD14_Msk);
   	SYS->GPD_MFP |= (SYS_GPD_MFP_PD15_BPWM0_CH4 | SYS_GPD_MFP_PD14_BPWM0_CH5);
		SYS->ALT_MFP3	&= ~(SYS_ALT_MFP3_PC0_Msk | SYS_ALT_MFP3_PC1_Msk | SYS_ALT_MFP3_PC2_Msk | SYS_ALT_MFP3_PC3_Msk | SYS_ALT_MFP3_PD15_Msk | SYS_ALT_MFP3_PD14_Msk);
		SYS->ALT_MFP3	|= (SYS_ALT_MFP3_PC0_BPWM0_CH0 | SYS_ALT_MFP3_PC1_BPWM0_CH1 | SYS_ALT_MFP3_PC2_BPWM0_CH2 | SYS_ALT_MFP3_PC3_BPWM0_CH3 | SYS_ALT_MFP3_PD15_BPWM0_CH4 | SYS_ALT_MFP3_PD14_BPWM0_CH5);
		
		/* Set GPB.8/12, GPD.7/6 and GPF.0/1 multi-function pins for BPWM1 Channel 0 ~ channel 5 */
    SYS->GPD_MFP &= ~(SYS_GPD_MFP_PD7_Msk | SYS_GPD_MFP_PD6_Msk);
   	SYS->GPD_MFP |= (SYS_GPD_MFP_PD7_BPWM1_CH0 | SYS_GPD_MFP_PD6_BPWM1_CH1);
		SYS->GPB_MFP &= ~(SYS_GPB_MFP_PB8_Msk | SYS_GPB_MFP_PB12_Msk);
   	SYS->GPB_MFP |= (SYS_GPB_MFP_PB8_BPWM1_CH2 | SYS_GPB_MFP_PB12_BPWM1_CH3);
		SYS->GPF_MFP &= ~(SYS_GPF_MFP_PF0_Msk | SYS_GPF_MFP_PF1_Msk);
		SYS->GPF_MFP |= (SYS_GPF_MFP_PF0_BPWM1_CH4 | SYS_GPF_MFP_PF1_BPWM1_CH5);
		SYS->ALT_MFP3	&= ~(SYS_ALT_MFP3_PD7_Msk | SYS_ALT_MFP3_PD6_Msk | SYS_ALT_MFP3_PB8_Msk | SYS_ALT_MFP3_PB12_Msk | SYS_ALT_MFP3_PF0_Msk | SYS_ALT_MFP3_PF1_Msk);
		SYS->ALT_MFP3	|= (SYS_ALT_MFP3_PD7_BPWM1_CH0 | SYS_ALT_MFP3_PD6_BPWM1_CH1 | SYS_ALT_MFP3_PB8_BPWM1_CH2 | SYS_ALT_MFP3_PB12_BPWM1_CH3 | SYS_ALT_MFP3_PF0_BPWM1_CH4 | SYS_ALT_MFP3_PF1_BPWM1_CH5);
		
		/* Set GPC.7 multi-function pins for GPIO to drive Bluetooth LED */
		SYS->GPC_MFP &= ~(SYS_GPC_MFP_PC7_Msk);
    SYS->GPC_MFP |= SYS_GPC_MFP_PC7_GPIO;
		/* Set GPB.2/3 multi-function pins for GPIO o drive Low Battery LED & Low Power LED */
		SYS->GPB_MFP &= ~(SYS_GPB_MFP_PB2_Msk | SYS_GPB_MFP_PB3_Msk);
    SYS->GPB_MFP |= (SYS_GPB_MFP_PB2_GPIO | SYS_GPB_MFP_PB3_GPIO);	
		
}

void UART0_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Configure UART0 and set UART0 Baudrate */
    UART_Open(UART0, 9600);
		/* Set RX FIFO Trigger Level to 4 bytes */
		UART0->FCR |= UART_FCR_RFITL_4BYTES;
	
		Command = 0;
		Bluetooth = 0;
	
}

/*---------------------------------------------------------------------------------------------------------*/
/* ISR to handle UART Channel 0 interrupt event                                                            */
/*---------------------------------------------------------------------------------------------------------*/
void UART02_IRQHandler(void)
{
    uint32_t IntSts = UART0->ISR;

    /* Get all the input characters */
    while(UART_IS_RX_READY(UART0))
    {
				/* Get the character from UART Buffer */
        UART_Read(UART0, buff, 4);
    }
		
		/* Bluetooth Connection LED contorl*/
		if((Bluetooth==0)&&(buff[3]==1))
		{
				Buzzer_Config(1);
				TIMER_Delay(TIMER0, 1000000);
				Buzzer_Config(0);
		}
		Bluetooth = buff[3];
		buff[3]?(BT_LED = On):(BT_LED = Off);
		
		/* Control Head LED brightness */
		HeadLED_Control(buff[2]);
		
		/* Get Action Command from buff[0] */
		if(buff[0] < 19)
			Command = buff[0];
		
		/* Get IR function Command from buff[1] */
		if(buff[1]==1)
				IR_Enable();
		else
				IR_Disable();
		
}

void PWM01_Init(void)
{
		/* Set PWM0/1 channel 0/2/4 prescale (each two channels share a prescaler) */
    PWM_SET_PRESCALER(PWM0, 0, 49);
		PWM_SET_PRESCALER(PWM0, 2, 49);
		PWM_SET_PRESCALER(PWM0, 4, 49);
	  PWM_SET_PRESCALER(PWM1, 0, 49);
		/* Set BPWM0/1 prescale (all 6 channels share a prescaler) */
		BPWM_SET_PRESCALER(BPWM0, 0, 49);
		BPWM_SET_PRESCALER(BPWM1, 0, 49);
    
	  /* Set PWM0/1 channel 0/2/4 as independent mode and down count type */
    PWM0->CTL1 = 0x0111;
		PWM1->CTL1 = 0x0111;
	  /* Set BPWM0/1 all 6 channels as down count type */
    BPWM0->CTL1 = 0x01;
		BPWM1->CTL1 = 0x01;
		
		/* Set PWM0/1 channel 0/2/4 Period */
    PWM_SET_CNR(PWM0, 0, 19999);
	  PWM_SET_CNR(PWM0, 2, 19999);
	  PWM_SET_CNR(PWM0, 4, 19999);
	  PWM_SET_CNR(PWM1, 0, 19999);
		/* Set BPWM0/1 all 6 channels Period */
    BPWM_SET_CNR(BPWM0, 0, 19999);
		BPWM_SET_CNR(BPWM1, 0, 19999);
	
		/* Set PWM0/1 PRDPCTLn = Output Low and CMPDCTLn = Output High */
	  PWM0->WGCTL0 = 0x05550000;
		PWM0->WGCTL1 = 0x0aaa0000;
		PWM1->WGCTL0 = 0x05550000;
		PWM1->WGCTL1 = 0x0aaa0000;
		/* Set BPWM0/1 PRDPCTLn = Output Low and CMPDCTLn = Output High */
	  BPWM0->WGCTL0 = 0x05550000;
		BPWM0->WGCTL1 = 0x0aaa0000;
		BPWM1->WGCTL0 = 0x05550000;
		BPWM1->WGCTL1 = 0x0aaa0000;
		
		/* Enable all PWM0/1 channels output */
    PWM_EnableOutput(PWM0, 0x3F);
	  PWM_EnableOutput(PWM1, 0x03);
		/* Enable all BPWM0/1 channels output */
    BPWM_EnableOutput(BPWM0, 0x3F);
	  BPWM_EnableOutput(BPWM1, 0x3F);
			
}

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, IP clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART to 9600-8n1 for receive message from Bluetooth*/
    UART0_Init();
		/* Enable UART0 interrupt*/
		UART_EnableInt(UART0, UART_IER_RDA_IEN_Msk);

		/* Init servo struct */
		SERVO_Init();
	
		/* Init PWM0/1 and BPWM0/1 */
	  PWM01_Init();

    /* Start all PWM0/1 counters */
    PWM_Start(PWM0, 0x15);
		PWM_Start(PWM1, 0x01);
		/* Start all BPWM0/1 counters */
    BPWM_Start(BPWM0, 0x01);
		BPWM_Start(BPWM1, 0x01);
		
		/* Init ADC channel4 & Timer1 for low battery detection */
		low_battery_Init();

		/* Init Buzzer function */
		Buzzer_Init();
		
		/* Init IR distance detect function */
		IR_Init();
// 		IR_Enable();
		
		/* Init Head Led Control function */
		HeadLED_Init();
		
	  /* Action Initial after power on*/
		Action_Init();
		/* Command Initial */
		Command = 0xFF;

		while(1)
		{	
			
				switch(Command)
				{
						/* Action Initial */
						case 0:
								Action_Init2();
								break;
						
						/* Forward(TouchDown) */
						case 1:
								Walk_Forward_TouchDown();
								break;
						/* Forward(Touch Up) */
						case 2:
								Walk_Forward_TouchUp();
								Command = 0xFF;
								break;
						
						/* Backward(TouchDown) */
						case 3:
								Walk_Backward_TouchDown();
								break;
						/* Backward(Touch Up) */
						case 4:
								Walk_Backward_TouchUp();
								Command = 0xFF;
								break;
						
						/* Left(TouchDown) */
						case 5:
								Shift_Left_TouchDown();
								break;
						/* Left(Touch Up) */
						case 6:
								Shift_Left_TouchUp();
								Command = 0xFF;
								break;
							
						/* Right(TouchDown) */
						case 7:
								Shift_Right_TouchDown();
								break;
						/* Right(Touch Up) */
						case 8:
								Shift_Right_TouchUp();
								Command = 0xFF;
								break;
							
						/* TurnLeft(TouchDown) */
						case 9:
								Turn_Left_TouchDown();
								break;
						/* TurnLeft(Touch Up) */
						case 10:
								Turn_Left_TouchUp();
								Command = 0xFF;
								break;

						/* TurnRight(TouchDown) */
						case 11:
								Turn_Right_TouchDown();
								break;
						/* TurnRight(Touch Up) */
						case 12:
								Turn_Right_TouchUp();
								Command = 0xFF;
								break;

						/* Open Hanger */
						case 13:
								Hanger_Open();
								break;
						/* Close Hanger */
						case 14:
								Hanger_Close();
								break;
						
						/* Action1 */
						case 15:
// 								Action1();
								/* Redo Action2, Action3 and Action4*/
								if(Command == 15)
								{
										Command = 16;
										Action2();
										if(Command == 16)
										{		
												Command = 17;
												Action3();
												if(Command == 17)
												{
														Command = 18;
														if(Command == 18)
														{
																Action4();
																Command = 15;
														}
												}
										}
								}
								break;
						/* Action2: Attack */
						case 16:
								Action2();
								Command = 0xFF;
								break;
						/* Action3: Twist body */
						case 17:
								Action3();
								Command = 0xFF;
								break;
						/* Action4: Dance */
						case 18:
								Action4();
								Command = 0xFF;
								break;
				
						case 255:
								break;
				}
				
		}
}

/*** (C) COPYRIGHT 2014 Nuvoton Technology Corp. ***/
